if select(5, GetAddOnInfo("PitBull4_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end

local PitBull4 = _G.PitBull4
if not PitBull4 then
	error("PitBull4_StaggerBar requires PitBull4")
end

-- CONSTANTS
local EXAMPLE_VALUE = 0.55
local DEFAULT_LIGHT_STAGGER_THRESHHOLD = 30
local DEFAULT_MODERATE_STAGGER_THRESHHOLD = 60

local THRESHHOLD_NONE = 0
local THRESHHOLD_LIGHT = 1
local THRESHHOLD_MODERATE = 2
local THRESHHOLD_HEAVY = 3

-- state
local light_stagger_threshhold = DEFAULT_LIGHT_STAGGER_THRESHHOLD
local moderate_stagger_threshhold = DEFAULT_MODERATE_STAGGER_THRESHHOLD
local current_threshhold = THRESHHOLD_NONE
local player_stagger_percent = 0

local L = PitBull4.L

local PitBull4_StaggerBar = PitBull4:NewModule("StaggerBar", "AceEvent-3.0")

local ChatFrame = getfenv(0)["DEFAULT_CHAT_FRAME"]
--ChatFrame:AddMessage("-Initialized-")

PitBull4_StaggerBar:SetModuleType("bar")
PitBull4_StaggerBar:SetName(L["Stagger bar"])
PitBull4_StaggerBar:SetDescription(L["Show a bar for the current stagger amount for Brewmaster Monks."])
PitBull4_StaggerBar.allow_animations = true
PitBull4_StaggerBar:SetDefaults({
	position = 4,
	light_stagger_threshhold = DEFAULT_LIGHT_STAGGER_THRESHHOLD,
	moderate_stagger_threshhold = DEFAULT_MODERATE_STAGGER_THRESHHOLD
})

function PitBull4_StaggerBar:OnEnable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "UpdateAll")
	self:RegisterEvent("UNIT_HEALTH_FREQUENT")
	self:RegisterEvent("UNIT_MAXHEALTH", "UNIT_HEALTH_FREQUENT")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")

	light_stagger_threshhold = self.db.profile.global.light_stagger_threshhold
	if light_stagger_threshhold == 0 then
		light_stagger_threshhold = DEFAULT_LIGHT_STAGGER_THRESHHOLD
		self.db.profile.global.light_stagger_threshhold = light_stagger_threshhold
	end
	moderate_stagger_threshhold = self.db.profile.global.moderate_stagger_threshhold
	if moderate_stagger_threshhold == 0 then
		moderate_stagger_threshhold = DEFAULT_MODERATE_STAGGER_THRESHHOLD
		self.db.profile.global.moderate_stagger_threshhold = moderate_stagger_threshhold
	end
end

function PitBull4_StaggerBar:GetValue(frame)
	local unit = frame.unit
	local class = UnitClass("player")
	local spec = GetSpecialization();
	local visible = unit == "player" and class == "Monk" and spec == 1;

	if not visible then
		return nil
	end

	return player_stagger_percent
end

function PitBull4_StaggerBar:GetExampleValue(frame)
	return EXAMPLE_VALUE
end

function PitBull4_StaggerBar:GetColor(frame, value)
	if value == nil or value < 0 then
		value = 0
	end
	if value > 100 then
		value = 100
	end

	local color = PitBull4.PowerColors.PB4_ALTERNATE
	if current_threshhold == THRESHHOLD_LIGHT then
		color = {0,1,0}
	elseif current_threshhold == THRESHHOLD_MODERATE then
		color = {1,1,0}
	elseif current_threshhold == THRESHHOLD_HEAVY then
		color = {1,0,0}
	end
	return unpack(color)
end
PitBull4_StaggerBar.GetExampleColor = PitBull4_StaggerBar.GetColor

function PitBull4_StaggerBar:UNIT_HEALTH_FREQUENT(_, unit)
	if unit == "player" then
		self:UpdatePlayerStaggerPercent()
		self:UpdateForUnitID(unit)
	end
end

function PitBull4_StaggerBar:ACTIVE_TALENT_GROUP_CHANGED()
	self:UpdateAll()
end

PitBull4_StaggerBar:SetGlobalOptionsFunction(function(self)
	return 'div', {
		type='header',
		name = '',
		desc = '',
	}, 'light_stagger_threshhold', {
		type = 'range',
		width = 'double',
		name = 'Light Stagger Threshhold',
		desc = 'Upper percentage bound for light stagger amount.',
		min = 1,
		max = 100,
		step = 1,
		get = function(info)
				local id = info[#info]
				local value = self:GetLightStaggerThreshholdValueOrDefault(self.db.profile.global[id])
				return value
		end,
		set = function(info, value)
				value = self:GetLightStaggerThreshholdValueOrDefault(value)
				local id = info[#info]
				self.db.profile.global[id] = value
				moderate_stagger_threshhold = value
		end,
	},
	'moderate_stagger_threshhold', {
			type = 'range',
			width = 'double',
			name = 'Moderate Stagger Threshhold',
			desc = 'Upper percentage bound for moderate stagger amount.',
			min = 1,
			max = 100,
			step = 1,
			get = function(info)
				local id = info[#info]
				local value = self:GetModerateStaggerThreshholdValueOrDefault(self.db.profile.global[id])
				return value
			end,
			set = function(info, value)
				value = self:GetModerateStaggerThreshholdValueOrDefault(value)
				local id = info[#info]
				self.db.profile.global[id] = value
				moderate_stagger_threshhold = value
			end,
		}
end);

function PitBull4_StaggerBar:GetLightStaggerThreshholdValueOrDefault(value)
	if value == nil or value < 1 or value > 100 then
		value = DEFAULT_LIGHT_STAGGER_THRESHHOLD
	end
	return value 
end

function PitBull4_StaggerBar:GetModerateStaggerThreshholdValueOrDefault(value)
	if value == nil or value < 1 or value > 100 then
		value = DEFAULT_MODERATE_STAGGER_THRESHHOLD
	end
	return value 
end

function PitBull4_StaggerBar:UpdateThreshhold()
	local value = player_stagger_percent * 100
	local newThreshhold = THRESHHOLD_NONE
	if value == nil or value < 0 then
		value = 0
	end
	if value > 100 then
		value = 100
	end

	if value == 0 then
		newThreshhold = THRESHHOLD_NONE
	elseif value <= self:GetLightStaggerThreshholdValueOrDefault(self.db.profile.global.light_stagger_threshhold) then
		newThreshhold = THRESHHOLD_LIGHT
	elseif value <= self:GetModerateStaggerThreshholdValueOrDefault(self.db.profile.global.moderate_stagger_threshhold) then
		newThreshhold = THRESHHOLD_MODERATE
	else
		newThreshhold = THRESHHOLD_HEAVY
	end

	if newThreshhold ~= current_threshhold then
		current_threshhold = newThreshhold
		self:UpdateAll()
	end
end

function PitBull4_StaggerBar:UpdatePlayerStaggerPercent()
	local stagger = UnitStagger("Player")
	local health = UnitHealthMax("Player")
	if stagger > health then
		stagger = health
	end

	player_stagger_percent = 1 - ((health - stagger) / health)
	self:UpdateThreshhold()
end
